/*
LCD drive
	Pin assignment
	RB7:O:LCD E
	RB6:O:LCD RS
	RA0:O:LCD D4
   	RA1:0:LCD D5
	RA2:O:LCD D6
	RA3:O:LCD D7
	LCD R/W is only VSS
	DB0 to DB3 are must be open
*/

#include <htc.h>
#include "main.h"
#include "lcd.h"


static bit LCD_RS	@ ((unsigned)&PORTB*8+6);	// Register select (RB6)
static bit LCD_EN	@ ((unsigned)&PORTB*8+7);	// Enable (RB7)


// write a byte to the LCD in 4 bit mode
void lcd_data_byteset(unsigned char c){
	PORTA = (PORTA & 0b11110000) |  (c & 0b00001111);		//Only low 4bit DATA
	LCD_STROBE;
}

//LCD data set
void lcd_data_set(unsigned char c){
	PORTA = (PORTA & 0b11110000) | ( (c >>4 ) & 0b00001111 ) ;	//Hi SIDE DATA
	LCD_STROBE;
	PORTA = (PORTA & 0b11110000) |  (c & 0b00001111 );		//low SIDE DATA
	LCD_STROBE;
}

//LCD write
void
lcd_write(unsigned char c)
{
	LCD_RS = 0;
	PORTA = (PORTA & 0b11110000) | ( (c >>4 ) & 0b00001111 ) ;	//Hi SIDE DATA
	LCD_STROBE;
	PORTA = (PORTA & 0b11110000) |  (c & 0b00001111 );		//low SIDE DATA
	LCD_STROBE;
	__delay_us(40);	
}

//Clear and home the LCD
void
lcd_clear(void)
{
	lcd_write(0x1);
	lcd_loccnt=0;
	__delay_ms(2); //PICC Pro
}

//write a string of chars to the LCD
void
lcd_puts(const char * s)
{
	LCD_RS = 1;	// write characters
	while(*s)
		lcd_putch(*s++);
}

//write one character to the LCD and auto increment location counter*/
void
lcd_putch(unsigned char c)
{
	LCD_RS = 1;	// write characters
	lcd_data_set(c);
	__delay_us(40);

	lcd_loccnt++;
	if (lcd_loccnt == LCD_LineLength1){
		lcd_loccnt = LCD_LineNumber2;
		lcd_goto(lcd_loccnt);
	}
	if (lcd_loccnt == LCD_LineLength2){
		lcd_loccnt = LCD_LineNumber1;
		lcd_goto(lcd_loccnt);
	}
}

//write to CG RAM 
void
lcd_cgwrite(unsigned char c)
{
	LCD_RS = 1;	// write characters
	lcd_data_set(c);
	__delay_us(40);
}

//Go to the specified position

void
lcd_goto(unsigned char pos)
{
	lcd_write(0x80+pos);
	lcd_loccnt=pos;
}
	
//initialise the LCD - put into 4 bit mode
void
lcd_init(void)
{
	LCD_RS = 0;				// write control bytes
	__delay_ms(15);			// power on delay (PICC Pro)
	lcd_data_byteset(0x03);	// attention!
	__delay_ms(5);			//wait
	LCD_STROBE;				//strobe again
	__delay_us(100);		//wait
	LCD_STROBE;				//strobe again
	__delay_ms(5);			//wait
	lcd_data_byteset(0x02) ;// set 4 bit mode
	__delay_us(40);			//wait
	lcd_write(0x28);		// 4 bit mode, 1/16 duty, 5x8 font
	lcd_write(0x08);		// display off
	lcd_write(0x0F);		// display on, blink curson on
	lcd_write(0x06);		// entry mode
	lcd_loccnt=0;
}

//Newline processing
void lcd_newline(unsigned char mode){
	unsigned char temp;
	switch (mode){
		case 1://the following are deleted
			if (lcd_loccnt <= LCD_LineLength1){
				lcd_space_full(LCD_LineLength1-lcd_loccnt);
				return;
			}else if (lcd_loccnt <= LCD_LineLength2) {
				lcd_space_full(LCD_LineLength2-lcd_loccnt);
				return;
			}else return;

		default://toggle the line
			if (lcd_loccnt <= LCD_LineLength1) lcd_goto(LCD_LineNumber2);
			else if (lcd_loccnt <= LCD_LineLength2) lcd_goto(LCD_LineNumber1);
	}
}	
void lcd_space_full(unsigned char count){//spacing
	while(count--){
		lcd_putch(' ');//space character write
	};
}		
		
